package net.blumia.pcmdroid.model

import androidx.lifecycle.LiveData
import androidx.room.*
import kotlinx.coroutines.flow.Flow

@Entity(tableName = "server_table")
data class Server (
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "serverId")
    val serverId: Long? = null,
    @ColumnInfo(name = "url")
    val url: String,
    @ColumnInfo(name = "name")
    val name: String,
    @ColumnInfo(name = "shortName")
    val shortName: String,
    @ColumnInfo(name = "baseFolderName")
    val baseFolderName: String,
) {
    fun displayName(): String {
        return name
    }

    fun shortDisplayName(): String {
        return if (shortName.isNotEmpty()) shortName else name
    }
}

@Dao
interface ServerDao {

    @Query("SELECT * FROM server_table") //  ORDER BY wtf ASC
    fun getServers(): Flow<List<Server>>

    @Query("SELECT * FROM server_table WHERE serverId=:id")
    fun getServerById(id: Long): Flow<Server>

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    suspend fun insert(server: Server)

    @Update
    suspend fun update(server: Server)

    @Delete
    suspend fun delete(server: Server)

    @Query("DELETE FROM server_table")
    suspend fun deleteAll()
}