package net.blumia.pcmdroid.model

import android.net.Uri
import android.util.Log
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject
import java.net.URLDecoder

data class Song (
    val filePath: String, // with file name
    val modifiedTime: Long = 0,
    val fileSize: Int = 0,
    val additionalInfo: Boolean = false, // if sidecar meta-info json file exist.
    val url: String,
) {
    fun displayName(): String {
        return Uri.parse(filePath).lastPathSegment ?: "???"
    }
}

data class Folder (
    val folderPath: String,
) {
    fun displayName(): String {
        return URLDecoder.decode(Uri.parse(folderPath).lastPathSegment ?: "???", "UTF-8")
    }
}

fun parseFromJsonString(jsonString: String): Pair<List<Folder>, List<Song>> {
    val folders = mutableListOf<Folder>()
    val songs = mutableListOf<Song>()

    try {
        val jsonObj = JSONObject(jsonString)
        val result = jsonObj.getJSONObject("result")
        val data = result.getJSONObject("data")
        val subFolders = data.getJSONArray("subFolderList")
        val musicList = if (data.has("musicList")) data.getJSONArray("musicList") else JSONArray()
        for (i in 0 until subFolders.length()) {
            val rawFolderName = subFolders.getString(i)
            folders.add(Folder(rawFolderName))
        }
        for (i in 0 until musicList.length()) {
            val fileObject = musicList.getJSONObject(i)
            val rawFileName = fileObject.getString("fileName")
            val fileName = URLDecoder.decode(rawFileName, "UTF-8")
            val modifiedTime = fileObject.getLong("modifiedTime")
            val fileSize = fileObject.getLong("fileSize")
            songs.add(
                Song(
                    rawFileName,
                    modifiedTime = modifiedTime,
                    url = rawFileName
                )
            )
        }
    } catch (e: JSONException) {
        e.printStackTrace()
        Log.d("vvv", jsonString)
    }

    return Pair(folders, songs)
}