package net.blumia.pcmdroid.ui.screen.addserver

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddCircle
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.rounded.AddCircle
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import net.blumia.pcmdroid.model.Server

@Composable
fun StepEnterApiUrl(
    urlStr: String = "",
    onUrlStrChange: (String) -> Unit = {},
) {
    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Icon(
            modifier = Modifier.size(50.dp),
            imageVector = Icons.Filled.AddCircle,
            contentDescription = null,
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            text = "Add Server",
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.h5,
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(bottom = 10.dp),
            text = "Add Private Cloud Music server",
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.subtitle2,
        )
        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            keyboardOptions = KeyboardOptions(
                keyboardType = KeyboardType.Uri,
            ),
            value = urlStr,
            onValueChange = onUrlStrChange,
            label = { Text("Server API Url") },
        )
    }
}

@Composable
fun StepOtherDetail(
    nameStr: String = "",
    onNameStrChange: (String) -> Unit = {},
    shortNameStr: String = "",
    onShortNameStrChange: (String) -> Unit = {},
    baseFolderNameStr: String = "",
    onBaseFolderNameStrChange: (String) -> Unit = {},
) {
    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = shortNameStr,
            onValueChange = onShortNameStrChange,
            label = { Text("Server short name") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = nameStr,
            onValueChange = onNameStrChange,
            label = { Text("Server name") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = baseFolderNameStr,
            onValueChange = onBaseFolderNameStrChange,
            label = { Text("Base folder name") },
        )
    }
}

@Preview(showBackground = true)
@Composable
fun AddServerScreen(
    onCloseActionTriggered: () -> Unit = {},
    onSubmitServerTriggered: (Server) -> Unit = {},
) {
    var stepState by remember { mutableStateOf(0) }
    val finalStep = 1

    fun onNextStepButtonTriggered() {
        if (stepState < 1) {
            stepState++
        }
    }

    fun onPreviousStepButtonTriggered() {
        if (stepState > 0) {
            stepState--
        }
    }

    Scaffold(
        topBar = {
            TopAppBar(
                backgroundColor = Color.Transparent,
                elevation = 0.dp,
                title = {},
                navigationIcon = {
                    IconButton(onClick = onCloseActionTriggered) {
                        Icon(imageVector = Icons.Filled.Close, contentDescription = "Close")
                    }
                },
            )
        }
    ) {
        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(20.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            var urlStr by rememberSaveable { mutableStateOf("") }
            var nameStr by rememberSaveable { mutableStateOf("") }
            var shortNameStr by rememberSaveable { mutableStateOf("") }
            var baseFolderNameStr by rememberSaveable { mutableStateOf("") }

            Column(
                modifier = Modifier
                    .fillMaxWidth()
                    .weight(1f)
            ) {

                when (stepState) {
                    0 -> {
                        StepEnterApiUrl(
                            urlStr,
                            onUrlStrChange = { value ->
                                urlStr = value
                            },
                        )
                    }
                    1 -> {
                        StepOtherDetail(
                            nameStr,
                            onNameStrChange = { value ->
                                nameStr = value
                            },
                            shortNameStr,
                            onShortNameStrChange = { value ->
                                shortNameStr = value
                            },
                            baseFolderNameStr,
                            onBaseFolderNameStrChange = { value ->
                                baseFolderNameStr = value
                            },
                        )
                    }
                }


            }

            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                TextButton(
                    onClick = ::onPreviousStepButtonTriggered,
                    enabled = stepState != 0,
                ) {
                    Text("Back")
                }
                Button(
                    onClick = {
                        if (stepState != finalStep) {
                            onNextStepButtonTriggered()
                        } else {
                            val srv = Server(
                                url = urlStr,
                                name = nameStr,
                                shortName = shortNameStr,
                                baseFolderName = baseFolderNameStr,
                            )
                            onSubmitServerTriggered(srv)
                        }
                    },
                    shape = RoundedCornerShape(50)
                ) {
                    Text(
                        text = if (stepState != finalStep) "Next" else "Done",
                        modifier = Modifier.padding(horizontal = 5.dp)
                    )
                }
            }
        }
    }
}